library super_sokets.stream_enhancer;

import "dart:async";
import "dart:collection";

class StreamEnhancer<T> {
  Queue<Completer<T>> _completers;
  Queue<T> _values;
  Stream<T> _stream;

  _addValue(T value) {
    _values.add(value);
    _update();
  }

  _update() {
    while (_completers.isNotEmpty && _values.isNotEmpty) {
      Completer<T> completer = _completers.removeFirst();
      T value = _values.removeFirst();
      completer.complete(value);
    }
  }

  Future<T> next() {
    var complete = new Completer<T>();
    _completers.add(complete);
    _update();
    return complete.future;
  }
      
  StreamEnhancer(Stream<T> stream) {
    this._completers = new Queue<Completer<T>>();
    this._values = new Queue<T>();
    this._stream = stream;
    this._stream.listen(_addValue);
  }
  
  get stream {
    return _stream;
  }
}

class WriteableStreamEnhancer<T> extends StreamEnhancer<T> {
  StreamController<T> _ctrl;
  
  WriteableStreamEnhancer(StreamController<T> ctrl) : super(ctrl.stream){
    this._ctrl = ctrl;
  }
  
  WriteableStreamEnhancer.fromScratch() : this(new StreamController<T>());
  
  void close() {
    _ctrl.close();
  }
  
  void add(T value) {
    _ctrl.add(value);
  }

  StreamController<T> get controller {
    return _ctrl;
  }
}
